/*
 *	parse.c		William R. Ward <hermit@ucscb.UCSC.EDU>
 *
 *	Given parameters (argc,argv) representing command-line
 *	arguments, function parse_args() returns an array of argument
 *	values.  The array is of type struct arg, which is described in
 *	the file parse.h.
 *
 *	The routine defines a argument as follows: all uppercase or
 *	lowercase letters in a word starting with '-' up to the first
 *	non-letter in the word, with any words not beginning with '-'
 *	or parts of a word after and including the last argument letter
 *	given as that argument's parameter.
 *
 *	For example, given the following strings in (argc, argv),
 *	parse_args recognizes the following as arguments:
 *
 *		input:		argument(s):	parameter:
 *		-r		r		(none)
 *		-abc		a		(none)
 *				b		(none)
 *				c		(none)
 *		-xy74		x		(none)
 *				y		74
 *		-z45h		z		45h
 *		-f filename	f		filename
 */

#include "parse.h"
#include <stdio.h>
#include <string.h>

#define TRUE 1
#define FALSE 0

extern void free();
extern char *calloc();
extern int atoi();
extern int sscanf();
extern int printf();

/***************************************************************************
  parse_args
****************************************************************************/

arg *
parse_args(int argc,char *argv[]) { 

  int i;
  static arg args[ MAXARGS ];
  char	*p,last = 0;

  for(i=1;i<argc;i++)
    if (*argv[i] == '-' && *(argv[i]+1) ) {
      for( p=argv[i]+1; chindex(*p); p++ )
	if (set_ch(args,*p))
	  last = *p;
      if ( *p )
	if ( p==argv[i]+1 )
	  set_str( args, p-1, last );
	else
	  set_str( args, p, last );
    } else {
      set_str( args, argv[i], last );
    }
  return args;
}

/***************************************************************************
  set_ch
****************************************************************************/

int
set_ch(arg args[],char x) {

  int index;

  index = chindex(x);
  if ( index ) {
    args[ index ].ch = 1;
    return 1;
  } else
    return 0;
}

/***************************************************************************
  set_str
****************************************************************************/

void
set_str( arg args[],char *x,char last ) {

  char	*t;
  int	index;
  
  index = chindex( last );
  if ( args[ index ].str == NULL ) {
    t = ( char * )calloc(strlen(x)+1,sizeof(char));
    strcpy(t,x);
  } else {
    t = (char *)calloc(strlen(args[index].str)+strlen(x)+1,sizeof(char));
    strcpy(t,args[index].str);
    strcat(t," ");
    strcat(t,x);
  }
  if (args[index].str != NULL) {
    free(args[index].str);
  }
  args[index].str = t;
}

/***************************************************************************
  GetBooleanOpt
****************************************************************************/

int 
GetBooleanOpt(arg *args,char c,int *var,int d) {

  int tmp;

  if ((d!=0) && (d!=1)) {
    printf("Warning: boolean option only takes 0 or 1 as the default\n\007");
    d=1;
  }
  if( !args[chindex(c)].ch ) {
    *var = d;
    return(FALSE);
  } else {
    if (args[chindex(c)].str==NULL) {
      *var = !d;
      return(TRUE);
    }
    tmp= atoi(args[chindex(c)].str);
    if ((tmp!=0)&&(tmp!=1)) {
      *var = !d;
      
    } else {
      *var = tmp;
    }
  }
  return(TRUE);
}

/***************************************************************************
  GetIntOpt
****************************************************************************/

int 
GetIntOpt(arg *args,char c,int *var,int d,int min,int max) {

  int tmp;

  if( !args[chindex(c)].ch ) {
    *var = d;
    return(FALSE);
  } else {
    if (args[chindex(c)].str==NULL) {
      *var = d;
      return(TRUE);
    }
    tmp = atoi(args[chindex(c)].str);
    if ((tmp<min) || (tmp>max)) {
      printf("Warning: integer option out of bound [%d,%d]\n\007",min,max);
      *var = d;
      return(FALSE);
    } else {
      *var = tmp;
      return(TRUE);
    }
  }
}

/***************************************************************************
  GetFloatOpt
****************************************************************************/

int 
GetFloatOpt(arg *args,char c,float *var,float d,float min,float max) {

  float tmp;

  if( !args[chindex(c)].ch ) {
    *var = d;
    return(FALSE);
  } else {
    sscanf(args[chindex(c)].str,"%f",&tmp);
    if ((tmp<min) || (tmp>max)) {
      printf("Warning: floating option out of bound [%f,%f]\n\007",min,max);
      *var = d;
      return(FALSE);
    } else {
      *var = tmp;
      return(TRUE);
    }
  }
}

/***************************************************************************
  GetStrOpt
****************************************************************************/

int 
GetStrOpt(arg *args,char c,char *var,char *d,int min,int max){

  char tmp[80];
  int len;

  if( !args[chindex(c)].ch ) {
    strcpy(var,d);
    return(FALSE);
  } else {
    strcpy(tmp,args[chindex(c)].str);
    len=strlen(tmp);
    if ((len<min) || (len>max)) {
      printf("Warning: string length out of bound [%d,%d]\n\007",min,max);
      strcpy(var,d);
      return(FALSE);
    } else {
      strcpy(var,tmp);
      return(TRUE);
    }
  }
}

